#
# Copyright 2025 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
from __future__ import annotations

from typing import List, Optional

from mypy_extensions import TypedDict
import trafaret as t

from datarobot._experimental.models.notebooks.enums import (
    KernelExecutionStatus,
    NotebookStatus,
    SessionType,
)
from datarobot.models.api_object import APIObject

# TODO: We're using trafaret's "ignore_extra" liberally and this is a subset of properties
notebook_session_trafaret = t.Dict(
    {
        t.Key("status"): t.String,
        t.Key("notebook_id"): t.String,
        t.Key("session_id"): t.String,
        t.Key("started_at", optional=True): t.String,
        t.Key("session_type", optional=True): t.String,
        t.Key("ephemeral_session_key", optional=True): t.String,
    }
).ignore_extra("*")


# TODO: We're using trafaret's "ignore_extra" - this is a subset of properties
notebook_execution_status_trafaret = t.Dict(
    {
        t.Key("status"): t.String,
        t.Key("cell_id", optional=True): t.String,
        t.Key("queued_cell_ids", optional=True): t.List(t.String),
    }
).ignore_extra("*")


class CloneRepositorySchema(TypedDict):
    url: str
    checkout_ref: Optional[str]


class StartSessionParameters(TypedDict):
    name: str
    value: str


class StartSessionPayload(TypedDict, total=False):
    is_triggered_run: bool
    parameters: Optional[List[StartSessionParameters]]
    open_file_paths: Optional[List[str]]
    clone_repository: Optional[CloneRepositorySchema]


class NotebookExecutionStatus(APIObject):
    """
    Notebook execution status information.

    Attributes
    ----------

    status : str
        The status of the notebook execution.
    cell_id : Optional[bson.ObjectId]
        The ID of the cell being executed. Optional.
    queued_cell_ids : Optional[List[bson.ObjectId]]
        The list of cell IDs that are queued for execution. Optional.
    """

    _converter = notebook_execution_status_trafaret

    def __init__(
        self,
        status: KernelExecutionStatus,
        cell_id: Optional[str] = None,
        queued_cell_ids: Optional[str] = None,
    ):
        self.status = status
        self.cell_id = cell_id
        self.queued_cell_ids = queued_cell_ids


class NotebookSession(APIObject):
    """
    Notebook session information.

    Attributes
    ----------

    status : NotebookStatus
        The current status of the notebook kernel.
    notebook_id : str
        The ID of the notebook.
    session_id : str
        The ID of the session. Incorporates the ``notebook_id`` as part of this ID.
    started_at : Optional[str]
        The date and time when the notebook was started. Optional.
    session_type: Optional[SessionType]
        The type of the run - either manual (triggered via UI or API) or scheduled. Optional.
    ephemeral_session_key: Optional[str]
        The ID specific to ephemeral session if being used. Optional.
    """

    _orchestrator_path = "api-gw/nbx/orchestrator/"

    _converter = notebook_session_trafaret

    def __init__(
        self,
        status: NotebookStatus,
        notebook_id: str,
        session_id: str,
        started_at: Optional[str] = None,
        session_type: Optional[SessionType] = None,
        ephemeral_session_key: Optional[str] = None,
    ):
        self.status = status
        self.notebook_id = notebook_id
        self.session_id = session_id
        self.started_at = started_at
        self.session_type = session_type
        self.ephemeral_session_key = ephemeral_session_key

    @classmethod
    def get(cls, notebook_id: str) -> NotebookSession:
        """
        Get a notebook session by its notebook ID.

        Parameters
        ----------
        notebook_id : str
            The ID of the notebook.

        Returns
        -------
        NotebookSession
            The notebook session information.
        """
        url = f"{cls._client.domain}/{cls._orchestrator_path}notebooks/{notebook_id}/"
        r_data = cls._client.get(url)
        return NotebookSession.from_server_data(r_data.json())

    @classmethod
    def start(cls, notebook_id: str, payload: StartSessionPayload) -> NotebookSession:
        """
        Start a notebook session.

        Parameters
        ----------
        notebook_id : str
            The ID of the notebook.
        payload : StartSessionPayload
            The payload to start the session.

        Returns
        -------
        NotebookSession
            The notebook session information.
        """
        url = f"{cls._client.domain}/{cls._orchestrator_path}notebooks/{notebook_id}/start/"
        r_data = cls._client.post(url, data=payload)
        return NotebookSession.from_server_data(r_data.json())

    @classmethod
    def stop(cls, notebook_id: str) -> NotebookSession:
        """
        Stop a notebook session.

        Parameters
        ----------
        notebook_id : str
            The ID of the notebook.

        Returns
        -------
        NotebookSession
            The notebook session information.
        """
        url = f"{cls._client.domain}/{cls._orchestrator_path}notebooks/{notebook_id}/stop/"
        r_data = cls._client.post(url)
        return NotebookSession.from_server_data(r_data.json())

    @classmethod
    def execute_notebook(cls, notebook_id: str, cell_ids: Optional[List[str]] = None) -> None:
        """
        Execute a notebook.

        Parameters
        ----------
        notebook_id : str
            The ID of the notebook.
        cell_ids : Optional[List[bson.ObjectId]]
            The list of cell IDs to execute. Optional. If not provided, the whole notebook will be executed.
        """
        url = f"{cls._client.domain}/{cls._orchestrator_path}notebooks/{notebook_id}/execute/"
        payload = {"cell_ids": cell_ids} if cell_ids else {}
        cls._client.post(url, data=payload)

    @classmethod
    def get_execution_status(cls, notebook_id: str) -> NotebookExecutionStatus:
        """
        Get the execution status information of a notebook.

        Parameters
        ----------
        notebook_id : str
            The ID of the notebook.

        Returns
        -------
        NotebookExecutionStatus
            The execution status information of the notebook.
        """
        url = (
            f"{cls._client.domain}/{cls._orchestrator_path}notebooks/{notebook_id}/executionStatus/"
        )
        r_data = cls._client.get(url)
        return NotebookExecutionStatus.from_server_data(r_data.json())
